@@ -21,11 +21,15 @@
 #include <unistd.h>
 #include <libgen.h>
 #include <libgnomevfs/gnome-vfs.h>
+#include <libgnomevfs/gnome-vfs-mime.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
+#include <libgnomevfs/gnome-vfs-mime-utils.h>
+#include <libgnomevfs/gnome-vfs-mime-info.h>
 #include <libgnomevfs/gnome-vfs-module-callback.h>
 #include <libgnomevfs/gnome-vfs-standard-callbacks.h>
 #include <libgnomevfs/gnome-vfs-xfer.h>
 #include <libgnomevfs/gnome-vfs-types.h>
+#include <libgnomevfs/gnome-vfs-application-registry.h>
 
 
 #include <gpe/init.h>
@@ -37,6 +41,7 @@
 #include <gpe/gpeiconlistview.h>
 #include <gpe/dirbrowser.h>
 #include <gpe/spacing.h>
+#include <gpe/gpehelp.h>
 
 #include "mime-sql.h"
 #include "mime-programs-sql.h"
@@ -55,6 +60,15 @@
 
 #define MAX_SYM_DEPTH 256
 
+#define N_(x) (x)
+
+#define HELPMESSAGE "GPE-Filemanager\nVersion " VERSION \
+		"\nGPE File Manager\n\ndctanner@magenet.com"\
+		"\nflorian@handhelds.org"
+
+#define NOHELPMESSAGE N_("Displaying help failed.")
+
+
 /* IDs for data storage fields */
 enum
 {
@@ -69,11 +83,22 @@ enum
 };
 static GtkTreeStore *store = NULL;
 
+enum
+{
+	COL_DIRICON,
+	COL_DIRNAME,
+	COL_DIRDATA,
+	N_DIRCOLUMNS
+};
+static GtkTreeStore *dirstore = NULL;
+
 static GtkWidget *window;
 static GtkWidget *combo;
 static GtkWidget *view_widget;
+static GtkWidget *dir_view_widget;
 static GtkWidget *view_window;
-static GtkWidget *vbox;
+static GtkWidget *dir_view_window;
+static GtkWidget *vbox, *main_hbox;
 
 static GtkWidget *bluetooth_menu_item;
 static GtkWidget *copy_menu_item;
@@ -83,16 +108,17 @@ static GtkWidget *move_menu_item;
 static GtkWidget *rename_menu_item;
 static GtkWidget *properties_menu_item;
 static GtkWidget *delete_menu_item;
+static GtkWidget *set_dirbrowser_menu_item;
+static GtkWidget *set_myfiles_menu_item;
 
 static GtkWidget *btnListView;
 static GtkWidget *btnIconView;
+static GtkWidget *btnGoUp;
 
 GdkPixbuf *default_pixbuf;
 
-guint screen_w, screen_h;
-
-GtkWidget *current_button=NULL;
-int current_button_is_down=0;
+GtkWidget *current_button = NULL;
+int current_button_is_down = 0;
 
 GtkItemFactory *item_factory;
 
@@ -109,12 +135,15 @@ gchar *current_directory = NULL;
 gchar *current_view = "icons";
 gint current_zoom = 36;
 static gboolean view_is_icons = FALSE;
+static gboolean directory_browser = TRUE;
+static gboolean limited_view = FALSE;
 
 static gchar *file_clipboard = NULL;
 
 static GHashTable *loaded_icons;
 static GtkWidget *progress_dialog = NULL;
 static gboolean abort_transfer = FALSE;
+static gboolean initialized = FALSE;
 
 typedef struct
 {
@@ -144,6 +173,8 @@ struct gpe_icon my_icons[] = {
 };
 
 
+/* some forward declarations */
+
 static void browse_directory (gchar *directory);
 static void popup_ask_open_with (void);
 static void popup_ask_move_file (void);
@@ -157,7 +188,13 @@ static void paste_file_clip (void);
 static void create_directory_interactive(void);
 static GtkWidget* create_view_widget_icons(void);
 static GtkWidget* create_view_widget_list(void);
+void on_about_clicked (GtkWidget * w);
+void on_help_clicked (GtkWidget * w);
+void on_dirbrowser_setting_changed(GtkCheckMenuItem *menuitem, gpointer user_data);
+void on_myfiles_setting_changed(GtkCheckMenuItem *menuitem, gpointer user_data);
 
+
+/* items of the context menu */
 static GtkItemFactoryEntry menu_items[] =
 {
   { "/Open Wit_h",	 NULL, popup_ask_open_with,  0, "<StockItem>", GTK_STOCK_OPEN },
@@ -175,6 +212,103 @@ static GtkItemFactoryEntry menu_items[] =
 
 static int nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);
 
+/* items of the main menu */
+static GtkItemFactoryEntry mMain_items[] = {
+  { N_("/_File"),         NULL,         NULL, 0, "<Branch>" },
+  { N_("/_File/_Create Directory"),NULL, create_directory_interactive, 0, "<Item>"},
+  { N_("/_File/s1"), NULL , NULL,    0, "<Separator>"},
+  { N_("/_File/_Close"),  NULL, gtk_main_quit, 0, "<StockItem>", GTK_STOCK_QUIT },
+  { N_("/_Settings"),         NULL,         NULL, 0, "<Branch>" },
+  { N_("/_Settings/Directory Browser"), NULL, on_dirbrowser_setting_changed, 0, "<CheckItem>"},
+  { N_("/_Settings/My Files only"), NULL, on_myfiles_setting_changed, 0, "<CheckItem>"},
+  { N_("/_Help"),         NULL,         NULL,           0, "<Branch>" },
+  { N_("/_Help/Index"),   NULL,         on_help_clicked,    0, "<StockItem>",GTK_STOCK_HELP },
+  { N_("/_Help/About"),   NULL,         on_about_clicked,    0, "<Item>" },
+};
+
+int mMain_items_count = sizeof(mMain_items) / sizeof(GtkItemFactoryEntry);
+
+/* create menu from description */
+GtkWidget *
+create_mMain(GtkWidget  *window)
+{
+  GtkItemFactory *itemfactory;
+  GtkAccelGroup *accelgroup;
+  
+  accelgroup = gtk_accel_group_new ();
+  
+  itemfactory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>",
+                                     accelgroup);
+  gtk_item_factory_create_items (itemfactory, mMain_items_count, 
+      mMain_items, NULL);
+  gtk_window_add_accel_group (GTK_WINDOW (window), accelgroup);
+  
+  set_dirbrowser_menu_item = 
+    gtk_item_factory_get_item (itemfactory, N_("/Settings/Directory Browser"));
+  set_myfiles_menu_item = 
+    gtk_item_factory_get_item (itemfactory, N_("/Settings/My Files only"));
+
+  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(set_dirbrowser_menu_item),
+                                 directory_browser); 
+  
+  return (gtk_item_factory_get_widget (itemfactory, "<main>"));
+}
+
+
+void
+show_message(GtkMessageType type, char* message)
+{
+	GtkWidget* dialog;
+	
+	dialog = gtk_message_dialog_new (GTK_WINDOW(window),
+					 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+					 type,
+					 GTK_BUTTONS_OK,
+					 message);
+	gtk_dialog_run (GTK_DIALOG(dialog));
+	gtk_widget_destroy(dialog);
+}
+
+void
+on_dirbrowser_setting_changed(GtkCheckMenuItem *menuitem, gpointer user_data)
+{
+  if (!initialized) 
+    return;
+  directory_browser = 
+    gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(set_dirbrowser_menu_item));
+  if (directory_browser)
+    gtk_widget_show_all(dir_view_window);
+  else
+	gtk_widget_hide(dir_view_window);
+  refresh_current_directory();
+}
+
+void
+on_myfiles_setting_changed(GtkCheckMenuItem *menuitem, gpointer user_data)
+{
+  gboolean enabled;
+  if (!initialized) 
+    return;
+  limited_view = 
+    gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(set_myfiles_menu_item));
+  enabled = (!limited_view || strcmp(current_directory,g_get_home_dir()));
+  gtk_widget_set_sensitive(btnGoUp,enabled);
+}
+
+void
+on_about_clicked (GtkWidget * w)
+{
+	show_message(GTK_MESSAGE_INFO,HELPMESSAGE);
+}
+
+
+void
+on_help_clicked (GtkWidget * w)
+{
+	if (gpe_show_help(PACKAGE,NULL))
+		show_message(GTK_MESSAGE_ERROR,NOHELPMESSAGE);
+}
+
 
 static void
 progress_response(GtkDialog *dialog, gint arg1, gpointer user_data)
@@ -845,19 +979,42 @@ ask_open_with (FileInformation *file_info)
   else
     applications = gnome_vfs_mime_get_all_applications (file_info->vfs->mime_type);
 
-  while (applications)
+  if (applications)
   {
-    application = (GnomeVFSMimeApplication *)(((GList *)applications)->data);
+    while (applications)
+    {
+      application = (GnomeVFSMimeApplication *)(((GList *)applications)->data);
 
-    row_text[0] = application->name;
-    gtk_clist_append (GTK_CLIST (clist), row_text);
-    gtk_clist_set_row_data (GTK_CLIST (clist), row_num, (gpointer) application);
+      row_text[0] = application->name;
+      gtk_clist_append (GTK_CLIST (clist), row_text);
+      gtk_clist_set_row_data (GTK_CLIST (clist), row_num, (gpointer) application);
 
-    printf ("Got application %s\n", ((GnomeVFSMimeApplication *)(((GList *)applications)->data))->command);
-    applications = applications->next;
-    row_num ++;
+      printf ("Got application %s\n", 
+		((GnomeVFSMimeApplication *)(((GList *)applications)->data))->command);
+      applications = applications->next;
+      row_num ++;
+    }
   }
+  else
+  {
+	GList *r_applications = 
+	  gnome_vfs_application_registry_get_applications(file_info->vfs->mime_type);
+	  
+    while (r_applications)
+    {
+      GnomeVFSMimeApplication *application = 
+	    gnome_vfs_application_registry_get_mime_application(r_applications->data);
+      row_text[0] = application->name;
+      gtk_clist_append (GTK_CLIST (clist), row_text);
+      gtk_clist_set_row_data (GTK_CLIST (clist), row_num, (gpointer) application);
 
+      printf ("Application from registry %s\n", 
+		((GnomeVFSMimeApplication *)(application)->command));
+      r_applications = r_applications->next;
+      row_num ++;
+    }
+  }
+  
 /*
   if (mime_programs)
   {
@@ -952,13 +1109,13 @@ button_clicked (GtkWidget *widget, gpointer udata)
 {
   GnomeVFSMimeApplication *default_mime_application;
   FileInformation *file_info;
-  gchar *command;
+  pid_t pid;
     
   file_info = (FileInformation *) udata;
 
   if (file_info == NULL)
     return;
-
+  
   /* Handle Symbolic Links -- CM */
   if (file_info->vfs->type == GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK) {
     GnomeVFSURI *uri = gnome_vfs_uri_new (file_info->vfs->symlink_name);
@@ -969,9 +1126,9 @@ button_clicked (GtkWidget *widget, gpointer udata)
       uri = gnome_vfs_uri_new (file_info->vfs->symlink_name);
       file_info->vfs = gnome_vfs_file_info_new ();
 
-      if (gnome_vfs_get_file_info_uri (uri, vfs, GNOME_VFS_FILE_INFO_DEFAULT) 
+      if (gnome_vfs_get_file_info_uri (uri, vfs, GNOME_VFS_FILE_INFO_DEFAULT)
           != GNOME_VFS_OK) {
-            printf ("Symbolic link %s leads to inaccessable file!\n", file_info->filename);
+            gpe_error_box_fmt (_("Symbolic link %s leads to inaccessable file!"), file_info->filename);
             return;
       }
 
@@ -979,12 +1136,14 @@ button_clicked (GtkWidget *widget, gpointer udata)
     } while (file_info->vfs->type == GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK && (--cur_depth > 0));
 
     if (cur_depth <= 0) {
-      printf ("max symbolic link depth exceeded for %s\n", file_info->filename);
+      gpe_error_box_fmt (_("Maximum symbolic link depth exceeded for %s"), file_info->filename);
       return;
     }
   }
   
-    if ((file_info->vfs->mime_type) && (!strcmp(file_info->vfs->mime_type,"application/x-gnome-app-info")))
+    if ((file_info->vfs->mime_type) && 
+		((!strcmp(file_info->vfs->mime_type,"application/x-desktop") && !file_info->vfs->size) 
+		 || strstr(file_info->vfs->mime_type,"x-directory/")))
         browse_directory (file_info->filename);
     else    
       if (file_info->vfs->type == GNOME_VFS_FILE_TYPE_REGULAR || file_info->vfs->type == GNOME_VFS_FILE_TYPE_UNKNOWN)
@@ -994,11 +1153,21 @@ button_clicked (GtkWidget *widget, gpointer udata)
           default_mime_application = gnome_vfs_mime_get_default_application (file_info->vfs->mime_type);
           if (default_mime_application != NULL)
           {
-            if (default_mime_application->requires_terminal)
-              command = g_strdup_printf (DEFAULT_TERMINAL " %s %s &", default_mime_application->command, file_info->filename);
-            else
-              command = g_strdup_printf ("%s %s &", default_mime_application->command, file_info->filename);
-            system (command);
+			pid = fork();
+			switch (pid)
+			{
+				case -1: 
+					return; /* failed */
+				break;
+				case  0: 
+				  if (default_mime_application->requires_terminal)
+					execlp(DEFAULT_TERMINAL,DEFAULT_TERMINAL,default_mime_application->command,file_info->filename);
+				  else
+					execlp(default_mime_application->command,default_mime_application->command,file_info->filename,NULL);
+				break;
+				default: 
+				break;
+			} 
           }
           else
             ask_open_with (file_info);
@@ -1011,14 +1180,14 @@ button_clicked (GtkWidget *widget, gpointer udata)
 }
 
 
-const gchar *
+gchar *
 find_icon_path (gchar *mime_type)
 {
   struct stat s;
   gchar *mime_icon;
   gchar *mime_path, *p;
 
-  mime_icon = gnome_vfs_mime_get_icon (mime_type);
+  mime_icon = g_strdup(gnome_vfs_mime_get_icon (mime_type));
   if (mime_icon)
   {
     if (mime_icon[0] == '/')
@@ -1080,7 +1249,7 @@ void
 add_icon (FileInformation *file_info)
 {
   GdkPixbuf *pixbuf = NULL;
-  const gchar *mime_icon;
+  gchar *mime_icon;
   
   if (file_info->vfs->type == GNOME_VFS_FILE_TYPE_DIRECTORY)
     mime_icon = g_strdup (PREFIX FILEMANAGER_ICON_PATH "/directory.png");
@@ -1098,30 +1267,55 @@ add_icon (FileInformation *file_info)
     mime_icon = g_strdup (PREFIX FILEMANAGER_ICON_PATH "/regular.png");
 
   pixbuf = get_pixbuf (mime_icon);
+  g_free(mime_icon);
   
-  /* now be careful */
-  if (view_is_icons)
-  	gpe_icon_list_view_add_item_pixbuf (GPE_ICON_LIST_VIEW (view_widget), file_info->vfs->name, pixbuf, (gpointer) file_info);
+  /* now be careful, we sort the items into the view widgets */
+  if ((directory_browser) && (file_info->vfs->type == GNOME_VFS_FILE_TYPE_DIRECTORY))
+    { /* add to directory browser */
+		GtkTreeIter iter;
+		GdkPixbuf *pb;
+		  
+		gtk_tree_store_append (dirstore, &iter, NULL);
+		pb = gdk_pixbuf_scale_simple(pixbuf,24,24,GDK_INTERP_BILINEAR);
+		
+		gtk_tree_store_set (dirstore, 
+		                    &iter,
+		                    COL_DIRICON, pb,
+			                COL_DIRNAME, file_info->vfs->name,
+			                COL_DIRDATA, (gpointer) file_info,
+			                -1);
+		gdk_pixbuf_unref(pb);
+    }
   else
-  {
-	GtkTreeIter iter;
-    GdkPixbuf *pb;
-	gchar *size;
-	  
-	pb = gdk_pixbuf_scale_simple(pixbuf,24,24,GDK_INTERP_BILINEAR);
-	size = gnome_vfs_format_file_size_for_display(file_info->vfs->size); 
-	gtk_tree_store_append (store, &iter, NULL);
-	  
-    gtk_tree_store_set (store, &iter,
-	    COL_NAME, file_info->vfs->name,
-	    COL_SIZE, size,
-	    COL_CHANGED, ctime(&file_info->vfs->ctime),
-		COL_ICON, pb,
-		COL_DATA, (gpointer) file_info,
-    	-1);
-	g_free(size);
-	gdk_pixbuf_unref(pb);
-  }	  
+    { /* add to iconlist or normal browser */
+	  if (view_is_icons)
+		gpe_icon_list_view_add_item_pixbuf (GPE_ICON_LIST_VIEW (view_widget), file_info->vfs->name, pixbuf, (gpointer) file_info);
+	  else
+	  {
+		GtkTreeIter iter;
+		GdkPixbuf *pb;
+		gchar *size;
+		gchar *time;
+		  
+		pb = gdk_pixbuf_scale_simple(pixbuf,24,24,GDK_INTERP_BILINEAR);
+		size = gnome_vfs_format_file_size_for_display(file_info->vfs->size); 
+		time = g_strdup(ctime(&file_info->vfs->ctime));
+		time[strlen(time)-1] = 0; /* strip newline */
+		  
+		gtk_tree_store_append (store, &iter, NULL);
+		
+		gtk_tree_store_set (store, &iter,
+			COL_NAME, file_info->vfs->name,
+			COL_SIZE, size,
+			COL_CHANGED, time,
+			COL_ICON, pb,
+			COL_DATA, (gpointer) file_info,
+			-1);
+		g_free(size);
+		g_free(time);
+		gdk_pixbuf_unref(pb);
+	  }
+  }  
 }
 
 gint
@@ -1167,6 +1361,8 @@ make_view ()
 		}while (gtk_tree_model_iter_next(GTK_TREE_MODEL(store),&iter));
 	}
     gtk_tree_store_clear(GTK_TREE_STORE(store));
+	if (directory_browser)
+        gtk_tree_store_clear(GTK_TREE_STORE(dirstore));
   }
   
   gtk_widget_draw (view_widget, NULL); // why?
@@ -1217,12 +1413,19 @@ make_view ()
     case GNOME_VFS_ERROR_READ_ONLY_FILE_SYSTEM:
       error = g_strdup ("Read only file system.");
       break;
+	case GNOME_VFS_ERROR_NOT_FOUND:
+	case GNOME_VFS_ERROR_GENERIC:
+	break;
     default:
-      error = g_strdup_printf ("Error: %s", gnome_vfs_result_to_string(result));
-      break;
+      error = g_strdup_printf ("Error: %s", 
+	                           gnome_vfs_result_to_string(open_dir_result));
+    break;
     }
-    gpe_error_box (error);
-    g_free (error);
+	if (error)
+	{
+	  gpe_error_box (error);
+      g_free (error);
+	}
   }  
   for (iter = list; iter; iter = iter->next)
     add_icon (iter->data);
@@ -1256,6 +1459,48 @@ add_history (gchar *directory)
 static void
 browse_directory (gchar *directory)
 {
+  gboolean enabled;
+  /* disable "up" button if we are in homedir and view is limited */
+  enabled = (!limited_view || strcmp(directory,g_get_home_dir()));
+  gtk_widget_set_sensitive(btnGoUp,enabled);
+  
+  /* some hacks to handle nasty smb urls */
+  if (g_str_has_prefix(directory,"smb:"))
+  {
+	int i, j, result;
+	GnomeVFSDirectoryHandle *handle;
+	char *mark = strstr(directory,"///");
+	if (mark)
+	{
+	   for (i=mark-directory+2;i<strlen(directory)-1;i++)
+		   directory[i] = directory[i+1];
+	   directory[strlen(directory)-1] = 0;
+	}
+	/* check if we can open it, if not strip workgroup */
+	if (strlen(directory) > 6) 
+	{
+		result = 
+			gnome_vfs_directory_open (&handle, directory, 
+								  GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE);
+		if (result != GNOME_VFS_OK)
+		{
+			if (result == GNOME_VFS_ERROR_NOT_A_DIRECTORY)
+			{
+				mark = strstr(directory+6,"/");
+				j = mark - (directory+5);
+				if (mark && (j>1))
+				{
+					for (i=5;i<strlen(directory)-j;i++)
+				   		directory[i] = directory[i+j];
+			   		directory[strlen(directory)-j] = 0;
+				}
+			}
+		}
+		else
+			gnome_vfs_directory_close(handle);
+	}
+  }
+
   if (current_directory) 
     g_free(current_directory);
   current_directory = g_strdup (directory);
@@ -1378,6 +1623,7 @@ tree_button_press (GtkWidget *tree,GdkEventButton *b, gpointer user_data)
       gint x, y;
       GtkTreeViewColumn *col;
       GtkTreePath *path;
+      GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(tree));
   
       x = b->x;
       y = b->y;
@@ -1392,7 +1638,7 @@ tree_button_press (GtkWidget *tree,GdkEventButton *b, gpointer user_data)
 
 	    gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &iter, path);
 
-	    gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, COL_DATA, &i, -1);
+	    gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, tree == view_widget ? COL_DATA : COL_DIRDATA, &i, -1);
 		
 	    gtk_tree_path_free (path);
         show_popup (NULL, i);
@@ -1423,19 +1669,18 @@ tree_button_release (GtkWidget *tree, GdkEventButton *b)
 					 NULL, NULL))
 	{
 	  GtkTreeIter iter;
+      GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(tree));
 	  FileInformation *i;
 
 	  gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &iter, path);
 
-	  gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, COL_DATA, &i, -1);
+	  gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, tree == view_widget ? COL_DATA : COL_DIRDATA, &i, -1);
 		
 	  gtk_tree_path_free (path);
 		
       button_clicked (NULL, i);
-
 	}
   }
-
   return TRUE;
 }
 
@@ -1495,7 +1740,7 @@ create_view_widget_list(void)
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(view_window),
   		GTK_POLICY_NEVER,GTK_POLICY_AUTOMATIC);
     gtk_container_add(GTK_CONTAINER(view_window),treeview);
-    gtk_box_pack_end (GTK_BOX (vbox), view_window, TRUE, TRUE, 0);
+    gtk_box_pack_end (GTK_BOX (main_hbox), view_window, TRUE, TRUE, 0);
 
 	g_signal_connect (G_OBJECT(treeview), "button_press_event", 
 		G_CALLBACK(tree_button_press), NULL);
@@ -1531,17 +1776,63 @@ create_view_widget_icons(void)
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(view_window),
   	GTK_POLICY_NEVER,GTK_POLICY_AUTOMATIC);
   gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(view_window),view_icons);
-  gtk_box_pack_end (GTK_BOX (vbox), view_window, TRUE, TRUE, 0);
+  gtk_box_pack_end (GTK_BOX (main_hbox), view_window, TRUE, TRUE, 0);
   
   gtk_widget_show_all(view_window);  
   return view_icons;
 }
 
 
+static GtkWidget*
+create_dir_view_widget(void)
+{
+    GtkWidget *treeview;
+	GtkCellRenderer *renderer;
+	GtkTreeViewColumn *column;
+	
+	treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (dirstore));
+  	gtk_tree_view_set_reorderable(GTK_TREE_VIEW(treeview),FALSE);
+	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW(treeview),TRUE);
+	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(treeview),TRUE);
+	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(treeview),TRUE);
+  
+	renderer = gtk_cell_renderer_pixbuf_new ();
+	g_object_set(renderer,"stock-size",GTK_ICON_SIZE_SMALL_TOOLBAR,NULL);
+	column = gtk_tree_view_column_new_with_attributes (_("Icon"),
+							   renderer,
+							   "pixbuf",
+							   COL_ICON,
+							   NULL);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
+	renderer = gtk_cell_renderer_text_new ();
+	column = gtk_tree_view_column_new_with_attributes (_("Directory"),
+							   renderer,
+							   "text",
+							   COL_DIRNAME,
+							   NULL);
+	gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column),TRUE);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
+
+    dir_view_window = gtk_scrolled_window_new(NULL,NULL);
+    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(dir_view_window),
+  		GTK_POLICY_NEVER,GTK_POLICY_AUTOMATIC);
+    gtk_container_add(GTK_CONTAINER(dir_view_window),treeview);
+    gtk_box_pack_start (GTK_BOX (main_hbox), dir_view_window, FALSE, TRUE, 0);
+
+	g_signal_connect (G_OBJECT(treeview), "button_press_event", 
+		G_CALLBACK(tree_button_press), NULL);
+	g_signal_connect (G_OBJECT(treeview), "button_release_event", 
+		G_CALLBACK(tree_button_release), NULL);
+	
+    gtk_widget_show_all(dir_view_window);
+	return treeview;
+}
+
+
 int
 main (int argc, char *argv[])
 {
-  GtkWidget *hbox, *toolbar, *toolbar2;
+  GtkWidget *hbox, *toolbar, *toolbar2, *mMain;
   GdkPixbuf *p;
   GtkWidget *pw;
   GtkAccelGroup *accel_group;
@@ -1556,22 +1847,30 @@ main (int argc, char *argv[])
 
   bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
   textdomain (PACKAGE);
-
+  
   bluetooth_init ();
 
-	/* init tree storage stuff */
-	store = gtk_tree_store_new (N_COLUMNS,
-  		G_TYPE_OBJECT,
-		G_TYPE_STRING,
-	 	G_TYPE_STRING,
-		G_TYPE_INT,
-		G_TYPE_STRING,
-		G_TYPE_STRING,
-		G_TYPE_POINTER);
+  /* init tree storage stuff */
+  store = gtk_tree_store_new (N_COLUMNS,
+                              G_TYPE_OBJECT,
+                              G_TYPE_STRING,
+                              G_TYPE_STRING,
+                              G_TYPE_INT,
+                              G_TYPE_STRING,
+                              G_TYPE_STRING,
+                              G_TYPE_POINTER);
   
+  dirstore = gtk_tree_store_new (N_DIRCOLUMNS,
+                                 G_TYPE_OBJECT,
+                                 G_TYPE_STRING,
+                                 G_TYPE_POINTER);
   /* main window */
   size_x = gdk_screen_width() / 2;
-  size_y = gdk_screen_height() * 2 / 3;  
+  size_y = gdk_screen_height() * 2 / 3;
+   /* if screen is large enough and landscape or if it is really large */
+  directory_browser = (((size_x > 240) && (size_y > 320) && (size_x > size_y))
+                      || (size_x > 600));
+  
   if (size_x < 240) size_x = 240;
   if (size_y < 320) size_y = 320;
   
@@ -1584,16 +1883,25 @@ main (int argc, char *argv[])
 
   gtk_widget_realize (window);
 
+  main_hbox = gtk_hbox_new (FALSE, gpe_get_boxspacing());
+  
   vbox = gtk_vbox_new (FALSE, 0);
 
   hbox = gtk_hbox_new (FALSE, 0);
 
+  /* main menu */ 
+  mMain = create_mMain(window);
+  gtk_box_pack_start(GTK_BOX(vbox),mMain,FALSE,TRUE,0);
+  
+  /* location stuff */
   combo = gtk_combo_new ();
   combo_signal_id = gtk_signal_connect (GTK_OBJECT (GTK_COMBO (combo)->entry),
     "activate", GTK_SIGNAL_FUNC (goto_directory), NULL);
   
   view_widget = create_view_widget_list();
   
+  dir_view_widget = create_dir_view_widget();
+  
   toolbar = gtk_toolbar_new ();
   gtk_toolbar_set_orientation (GTK_TOOLBAR (toolbar), GTK_ORIENTATION_HORIZONTAL);
 
@@ -1608,19 +1916,19 @@ main (int argc, char *argv[])
 			    _("Forward"), _("Go forward in history."),
 			    G_CALLBACK (history_forward), NULL, -1);
 
-  gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_GO_UP,
-			    _("Up one level"), _("Go up one level."),
-			    G_CALLBACK (up_one_level), NULL, -1);
+  btnGoUp = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_GO_UP,
+			                          _("Up one level"), _("Go up one level."),
+			                          G_CALLBACK (up_one_level), NULL, -1);
 
   gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
 
-  gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_STOP,
-			    _("Stop"), _("Stop the current process."),
-			    G_CALLBACK (safety_check), NULL, -1);
-
   gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_HOME,
 			    _("Home"), _("Goto your home directory."),
 			    G_CALLBACK (set_directory_home), NULL, -1);
+                
+  gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_STOP,
+			    _("Stop"), _("Stop the current process."),
+			    G_CALLBACK (safety_check), NULL, -1);
 
   gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
 
@@ -1637,17 +1945,18 @@ main (int argc, char *argv[])
 			   G_CALLBACK (view_list), NULL);
   gtk_widget_set_sensitive(btnListView,FALSE);
                
-  p = gpe_find_icon ("dir-up");
-  pw = gtk_image_new_from_pixbuf(p);
-  gtk_toolbar_append_item (GTK_TOOLBAR (toolbar2), _("Goto Location"), 
+  pw = gtk_image_new_from_stock(GTK_STOCK_JUMP_TO, 
+                                gtk_toolbar_get_icon_size(GTK_TOOLBAR(toolbar2)));
+  gtk_toolbar_append_item (GTK_TOOLBAR (toolbar2), _("Go!"), 
 			   _("Goto Location"), _("Goto Location"), pw, 
 			   G_CALLBACK (goto_directory), NULL);
-
+  
   gtk_container_add (GTK_CONTAINER (window), GTK_WIDGET (vbox));
   gtk_box_pack_start (GTK_BOX (vbox), toolbar, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (hbox), combo, TRUE, TRUE, 0);
   gtk_box_pack_start (GTK_BOX (hbox), toolbar2, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), main_hbox, TRUE, TRUE, 0);
 
   gpe_set_window_icon (window, "icon");
 
@@ -1675,9 +1984,26 @@ main (int argc, char *argv[])
 						  (GnomeVFSModuleCallback) auth_callback,
 						  NULL,
 						  NULL);
-                    
-  set_directory_home (NULL);
-
+						  
+  gnome_vfs_mime_info_reload();
+  gnome_vfs_application_registry_reload();
+  
+  if (argc < 2)
+    set_directory_home (NULL);
+  else
+    {
+      GnomeVFSResult res;
+      GnomeVFSDirectoryHandle *handle;
+      res = gnome_vfs_directory_open(&handle, argv[1], GNOME_VFS_FILE_INFO_DEFAULT);
+      if (res == GNOME_VFS_OK)
+        {
+          gnome_vfs_directory_close(handle);
+          browse_directory (argv[1]);
+        }
+      else
+        set_directory_home (NULL);
+    }
+  initialized = TRUE;  
   gtk_main();
 
   return 0;
